searchData={"items":[{"type":"module","doc":"`Estructura` is a set of extensions for Elixir structures,\n  such as `Access` implementation, `Enumerable` and `Collectable`\n  implementations, validations and test data generation via `StreamData`.\n\n`Estructura` simplifies the following\n\n  * `Access` implementation for structs\n  * `Enumerable` implementation for structs (as maps)\n  * `Collectable` implementation for one of struct’s fields (as `MapSet` does)\n  * `StreamData` generation of structs for property-based testing","title":"Estructura","ref":"Estructura.html"},{"type":"module","doc":"`use Estructura` accepts four keyword arguments.\n\n  * `access: true | false | :lazy` whether to generate the `Access` implementation, default `true`;\n    when `true` or `:lazy`, it also produces `put/3` and `get/3` methods to be used with `coercion`\n    and `validation`, when `:lazy`, instances of `Estructura.Lazy` are understood as values\n  * `coercion: boolean() | [key()]` whether to generate the bunch of `coerce_×××/1` functions\n    to be overwritten by implementations, default `false`\n  * `validation: boolean() | [key()]` whether to generate the bunch of `validate_×××/1` functions\n    to be overwritten by implementations, default `false`\n  * `calculated: [{key(), formula}] when formula: binary() | Formulae.t() | (t() -> any())` the calculated fields \n  * `enumerable: boolean()` whether to generate the `Enumerable` porotocol implementation, default `false`\n  * `collectable: false | key()` whether to generate the `Collectable` protocol implementation,\n    default `false`; if non-falsey atom is given, it must point to a struct field where `Collectable`\n    would collect. Should be one of `list()`, `map()`, `MapSet.t()`, `bitstribg()`\n  * `generator: %{optional(key()) => Estructura.Config.generator()}` the instructions\n    for the `__generate__/{0,1}` functions that would produce the target structure values suitable\n    for usage in `StreamData` property testing; the generated `__generator__/1` function is overwritable.\n\nPlease note, that setting `coercion` and/or `validation` to truthy values has effect\n  if and only if `access` has been also set to `true`.\n\nTypical example of usage would be:\n\n```elixir\ndefmodule MyStruct do\n  use Estructura,\n    access: true,\n    coercion: [:foo], # requires `c:MyStruct.Coercible.coerce_foo/1` impl\n    validation: true, # requires `c:MyStruct.Validatable.validate_×××/1` impls\n    calculated: [foo: \"length(bar)\"], # requires `:formulae` dependency\n    enumerable: true,\n    collectable: :bar,\n    generator: [\n      foo: {StreamData, :integer},\n      bar: {StreamData, :list_of, [{StreamData, :string, [:alphanumeric]}]},\n      baz: {StreamData, :fixed_map,\n        [[key1: {StreamData, :integer}, key2: {StreamData, :integer}]]}\n    ]\n\n  defstruct foo: 0, bar: [], baz: %{}\n\n  @impl MyStruct.Coercible\n  def coerce_foo(value) when is_integer(value), do: {:ok, value}\n  def coerce_foo(value) when is_float(value), do: {:ok, round(value)}\n  def coerce_foo(value) when is_binary(value) do\n    case Integer.parse(value) do\n      {value, \"\"} -> {:ok, value}\n      _ -> {:error, \"#{value} is not a valid integer value\"}\n    end\n  end\n  def coerce_foo(value),\n    do: {:error, \"Cannot coerce value given for `foo` field (#{inspect(value)})\"}\n\n  @impl MyStruct.Validatable\n  def validate_foo(value) when value >= 0, do: {:ok, value}\n  def validate_foo(_), do: {:error, \":foo must be positive\"}\n\n  @impl MyStruct.Validatable\n  def validate_bar(value), do: {:ok, value}\n\n  @impl MyStruct.Validatable\n  def validate_baz(value), do: {:ok, value}\nend\n```\n\nThe above would allow the following to be done with the structure:\n\n```elixir\ns = %MyStruct{}\n\nput_in s, [:foo], \"42\"\n#⇒ %MyStruct{foo: 42, bar: [], baz: %{}}\n\nfor i <- [1, 2, 3], into: s, do: i\n#⇒ %MyStruct{foo: 0, bar: [1, 2, 3], baz: %{}}\n\nEnum.map(s, &elem(&1, 1))\n#⇒ [0, [], %{}]\n\nMyStruct.__generator__() |> Enum.take(3)\n#⇒ [\n#      %MyStruct{bar: [], baz: %{key1: 0, key2: 0}, foo: -1},\n#      %MyStruct{bar: [\"g\", \"xO\"], baz: %{key1: -1, key2: -2}, foo: 2},\n#      %MyStruct{bar: [\"\", \"\", \"\"], baz: %{key1: -3, key2: 1}, foo: -1}\n#    ]\n```","title":"Use Options - Estructura","ref":"Estructura.html#module-use-options"},{"type":"module","doc":"When using `Access`, the calculated fields would be also updated upon the\n  update the fields then depend on.","title":"Calculated fields - Estructura","ref":"Estructura.html#module-calculated-fields"},{"type":"module","doc":"When `coercion: true | [key()]` is passed as an argument to `use Estructura`,\nthe nested behaviour `Coercible` is generated and the target module claims to implement it.\n\nTo make a coercion work with `MyStruct.put/3` and `put_in/3` provided\nby `Access` implementation, the consumer module should implement `MyStruct.Coercible`\nbehaviour.\n\nFor the consumer convenience, the warnings for not implemented functions will be issued by compiler.","title":"Coercion - Estructura","ref":"Estructura.html#module-coercion"},{"type":"module","doc":"When `validation: true | [key()]` is passed as an argument to `use Estructura`,\nthe nested behaviour `Validatable` is generated and the target module claims to implement it.\n\nTo make a validation work with `MyStruct.put/3` and `put_in/3` provided\nby `Access` implementation, the consumer module should implement `MyStruct.Validatable`\nbehaviour.\n\nFor the consumer convenience, the warnings for not implemented functions will be issued by compiler.","title":"Validation - Estructura","ref":"Estructura.html#module-validation"},{"type":"module","doc":"If `generator` keyword argument has been passed, `MyStruct.__generate__/{0,1}` can be\nused to generate instances of this struct for `StreamData` property based tests.\n\n```elixir\nproperty \"generation\" do\n  check all %MyStruct{foo: foo, bar: bar, baz: baz} <- MyStruct.__generator__() do\n    assert match?(%{key1: v1, key2: v2} when is_integer(v1) and is_integer(v2), baz)\n    assert is_integer(foo)\n    assert is_binary(bar)\n  end\nend\n```","title":"Generation - Estructura","ref":"Estructura.html#module-generation"},{"type":"module","doc":"If `access: :lazy` is passed as an option, the struct content might be instantiated lazily,\nupon first access through `Kernel.×××_in/{2,3}` family.\n\nThis might be explicitly helpful when the real content requires a significant time\nto load and/or store. Consider the full response from the web server, including\nthe gzipped content, which might in turn be a huge text file. Or an attachment to an email.\n\nInstead of unarchiving the content, one might use `Lazy` as\n\n```elixir\ndefmodule Response do\n  @moduledoc false\n  use Estructura, access: :lazy\n\n  def extract(file), do: {:ok, ZipHelper.unzip(file)}\n\n defstruct __lazy_data__: nil,\n   file: Estructura.Lazy.new(&Response.extract/1)\nend\n\nresponse = %Response{__lazy_data__: zipped_content}\n# immediate response\n\nresponse |> get_in([:file])\n# unzip and return\n\n{unzipped, struct_with_cached_value} = response |> pop_in([:file])\n# unzip and return the value, alter the struct with it\n```\n\nSee `Estructura.Lazy` for details and options, see `Estructura.LazyMap` for\nthe implementation of lazy map.","title":"Lazy - Estructura","ref":"Estructura.html#module-lazy"},{"type":"function","doc":"Instantiates the struct by using `Access` from a map, passing all coercions and validations.","title":"Estructura.coerce/3","ref":"Estructura.html#coerce/3"},{"type":"function","doc":"Calculates the difference between two estructures and returns a tuple with\n  the first element containing same values and the second one with diffs.\n\nThis function accepts maps but this options should be used as a last resort\nbecause structs are 4–6 times faster.","title":"Estructura.diff/3","ref":"Estructura.html#diff/3"},{"type":"function","doc":"```elixir\ndefmodule M do\n  use Estructura, enumerable: true\n  defstruct a: true, b: false\nend\nEstructura.diff(struct(M, []), struct(M, b: true), :diff)\n#⇒{%{a: true}, %{b: [false, true]}}\n\nEstructura.diff(%{a: true, b: false}, %{a: true, b: true}, :overlap)\n#⇒ %{a: true}\n\nEstructura.diff(%{a: true, b: false}, %{a: true, b: true}, :disjoint)\n#⇒ %{b: [false, true]}\n```","title":"Examples - Estructura.diff/3","ref":"Estructura.html#diff/3-examples"},{"type":"type","doc":"Diff return type","title":"Estructura.diff_result/0","ref":"Estructura.html#t:diff_result/0"},{"type":"module","doc":"The implementation of `Estructura` ready to work with tree AST-like structure","title":"Estructura.Aston","ref":"Estructura.Aston.html"},{"type":"function","doc":"See `Estructura.Aston.__generator__/1`","title":"Estructura.Aston.__generator__/0","ref":"Estructura.Aston.html#__generator__/0"},{"type":"function","doc":"Returns the generator to be used in `StreamData`-powered property testing, based\n  on the specification given to `use Estructura`, which contained","title":"Estructura.Aston.__generator__/1","ref":"Estructura.Aston.html#__generator__/1"},{"type":"function","doc":"```elixir\n%Estructura.Config{\n  access: true,\n  coercion: true,\n  validation: true,\n  calculated: [],\n  collectable: :content,\n  enumerable: true,\n  generator: [\n    name: {StreamData, :string, [:alphanumeric]},\n    attributes: {StreamData, :map_of,\n     [\n       {StreamData, :atom, [:alphanumeric]},\n       {StreamData, :one_of,\n        [\n          [\n            {StreamData, :integer},\n            {StreamData, :boolean},\n            {StreamData, :string, [:alphanumeric]}\n          ]\n        ]}\n     ]},\n    content: {StreamData, :tree,\n     [{StreamData, :fixed_list, [[]]}, &Estructura.Aston.child_gen/1]}\n  ]\n}\n```\n\n\nThe argument given would be used as a template to generate new values.","title":"shape - Estructura.Aston.__generator__/1","ref":"Estructura.Aston.html#__generator__/1-shape"},{"type":"function","doc":"Returns the key to be used for accessing the nested element(s)","title":"Estructura.Aston.access/2","ref":"Estructura.Aston.html#access/2"},{"type":"function","doc":"Coerces the deeply nested map to an instance of nested `Estructura.Aston`","title":"Estructura.Aston.coerce/3","ref":"Estructura.Aston.html#coerce/3"},{"type":"function","doc":"Gets the value for the given key from the structure","title":"Estructura.Aston.get/3","ref":"Estructura.Aston.html#get/3"},{"type":"function","doc":"Puts the value for the given key into the structure, passing coercion _and_ validation,\n  returns `{:ok, updated_struct}` or `{:error, reason}` if there is no such key","title":"Estructura.Aston.put/3","ref":"Estructura.Aston.html#put/3"},{"type":"function","doc":"Puts the value for the given key into the structure, passing coercion _and_ validation,\n  returns the value or raises if there is no such key","title":"Estructura.Aston.put!/3","ref":"Estructura.Aston.html#put!/3"},{"type":"function","doc":"","title":"Estructura.Aston.recalculate_calculated/1","ref":"Estructura.Aston.html#recalculate_calculated/1"},{"type":"type","doc":"","title":"Estructura.Aston.t/0","ref":"Estructura.Aston.html#t:t/0"},{"type":"function","doc":"Converts `Estructura.Aston` to the XML AST understandable by `XmlBuilder`","title":"Estructura.Aston.to_ast/1","ref":"Estructura.Aston.html#to_ast/1"},{"type":"module","doc":"Default coercer for `:datetime`, coercing strings (_ISO8601_) and integers (_epoch_)","title":"Estructura.Coercers.DateTime","ref":"Estructura.Coercers.DateTime.html"},{"type":"module","doc":"The nested struct with helpers to easily describe it and produce\n  validation, coercion, and generation helpers.","title":"Estructura.Nested","ref":"Estructura.Nested.html"},{"type":"macro","doc":"DSL helper to produce **`coerce`** callbacks. The syntax is kinda weird,\n  but bear with it, please.\n\nIt’s known to produce warnings in `credo`, I’m working on it.\n```elixir\ncoerce do\n  def data.age(age) when is_float(age), do: {:ok, age}\n  def data.age(age) when is_integer(age), do: {:ok, 1.0 * age}\n  def data.age(age) when is_binary(age), do: {:ok, String.to_float(age)}\n  def data.age(age), do: {:error, \"Could not cast #{inspect(age)} to float\"}\nend\n```","title":"Estructura.Nested.coerce/1","ref":"Estructura.Nested.html#coerce/1"},{"type":"macro","doc":"","title":"Estructura.Nested.init/1","ref":"Estructura.Nested.html#init/1"},{"type":"macro","doc":"Declares the shape of the target nested map. the values might be:\n\n- `:string` | `:integer` or another simple `type` understood by\n  [`StreamData`](https://hexdocs.pm/stream_data/StreamData.html#functions)\n- `[type]` to declare a list of elements of a single `type`\n- `map` to declare a nesting level; in such a case, the module with the FQN\n  is created, carrying the struct of the same behaviour\n- `mfa` tuple pointing out to the generator for this value","title":"Estructura.Nested.shape/1","ref":"Estructura.Nested.html#shape/1"},{"type":"macro","doc":"```elixir\ndefmodule User do\n  use Estructura.Nested\n  shape %{\n    name: :string,\n    address: %{city: :string, street: %{name: [:string], house: :string}},\n    data: %{age: :float}\n  }\nend\n\n%User{}\n```\n\nwould result in\n\n```elixir\n%User{\n  address: %User.Address{\n    city: nil,\n    street: %User.Address.Street{house: nil, name: []}\n  },\n  data: %User.Data{age: nil},\n  name: nil\n  }\n```","title":"Example - Estructura.Nested.shape/1","ref":"Estructura.Nested.html#shape/1-example"},{"type":"macro","doc":"DSL helper to produce **`validate`** callbacks. The syntax is kinda weird,\n  but bear with it, please.\n\nIt’s known to produce warnings in `credo`, I’m working on it.\n```elixir\nvalidate do\n  def address.street.postal_code(<<?0, code::binary-size(4)>>),\n    do: {:ok, code}\n  def address.street.postal_code(code),\n    do: {:error, \"Not a postal code (#{inspect(code)})\"}\nend\n```","title":"Estructura.Nested.validate/1","ref":"Estructura.Nested.html#validate/1"},{"type":"behaviour","doc":"The type to be used for coercing, validating, and generation\n  of the implementation’s instances.","title":"Estructura.Nested.Type","ref":"Estructura.Nested.Type.html"},{"type":"callback","doc":"Coerces the value coming from outside","title":"Estructura.Nested.Type.coerce/1","ref":"Estructura.Nested.Type.html#c:coerce/1"},{"type":"callback","doc":"The generator for the type","title":"Estructura.Nested.Type.generate/0","ref":"Estructura.Nested.Type.html#c:generate/0"},{"type":"callback","doc":"The generator for the type accepting options","title":"Estructura.Nested.Type.generate/1","ref":"Estructura.Nested.Type.html#c:generate/1"},{"type":"callback","doc":"Validates the value as being correct","title":"Estructura.Nested.Type.validate/1","ref":"Estructura.Nested.Type.html#c:validate/1"},{"type":"module","doc":"`Estructura` type for `Date`","title":"Estructura.Nested.Type.Date","ref":"Estructura.Nested.Type.Date.html"},{"type":"module","doc":"`Estructura` type for `Date`","title":"Estructura.Nested.Type.DateTime","ref":"Estructura.Nested.Type.DateTime.html"},{"type":"module","doc":"`Estructura` type for a limited set of values, might be used as an implementation generator.","title":"Estructura.Nested.Type.Enum","ref":"Estructura.Nested.Type.Enum.html"},{"type":"module","doc":"`Estructura` type for `Date`","title":"Estructura.Nested.Type.IP","ref":"Estructura.Nested.Type.IP.html"},{"type":"macro","doc":"","title":"Estructura.Nested.Type.IP.sigil_IP/2","ref":"Estructura.Nested.Type.IP.html#sigil_IP/2"},{"type":"type","doc":"","title":"Estructura.Nested.Type.IP.t/0","ref":"Estructura.Nested.Type.IP.html#t:t/0"},{"type":"module","doc":"`Estructura` type for a set of predefined values, might be used as an implementation generator.","title":"Estructura.Nested.Type.Tags","ref":"Estructura.Nested.Type.Tags.html"},{"type":"module","doc":"`Estructura` type for `Date`","title":"Estructura.Nested.Type.Time","ref":"Estructura.Nested.Type.Time.html"},{"type":"module","doc":"`Estructura` type for `Date`","title":"Estructura.Nested.Type.URI","ref":"Estructura.Nested.Type.URI.html"},{"type":"protocol","doc":"","title":"Estructura.Flattenable","ref":"Estructura.Flattenable.html"},{"type":"function","doc":"The function returning the flattened input.\n\nThis protocol is explicitly handful when deeply nested structs are to be serialized.\n\n```elixir\niex|%_{}|1 ▶ %Estructura.User{}\n%Estructura.User{\n  address: %Estructura.User.Address{\n    city: nil,\n    street: %Estructura.User.Address.Street{house: nil, name: []}\n  },\n  data: %Estructura.User.Data{age: nil},\n  name: nil\n}\n\niex|%_{}|2 ▶ Estructura.Flattenable.flatten(%Estructura.User{}, coupler: \"-\", except: ~w|address-street data-age|)\n%{\"address-city\" => nil, \"birthday\" => nil, \"created_at\" => nil, \"name\" => nil}\niex|%_{}|3 ▶ Estructura.Flattenable.flatten(%Estructura.User{}, only: ~w|address_street data_age|)\n%{\"address_street_house\" => nil, \"data_age\" => nil}\niex|%_{}|4 ▶ Estructura.Flattenable.flatten(%Estructura.User{}, only: ~w|address|)\n%{\"address_city\" => nil, \"address_street_house\" => nil}\n```\n\nAllowed options are:\n\n- **`coupler`** the string to concatenate nested keys with, _default:_ **`-`** \n- **`only`** the list of keys to select\n- **`except`** the list of keys to ignore\n- **`jsonify`** `true` or a json encoder implementation; if set, the values will be jsonified\n\nTo enable it for your struct, use `@derive Estructura.Flattenable` or\n  `@derive {Estructura.Flattenable, options}`. `Estructura` implementations derive it be default.","title":"Estructura.Flattenable.flatten/2","ref":"Estructura.Flattenable.html#flatten/2"},{"type":"type","doc":"All the types that implement this protocol.","title":"Estructura.Flattenable.t/0","ref":"Estructura.Flattenable.html#t:t/0"},{"type":"protocol","doc":"","title":"Estructura.Transformer","ref":"Estructura.Transformer.html"},{"type":"type","doc":"All the types that implement this protocol.","title":"Estructura.Transformer.t/0","ref":"Estructura.Transformer.html#t:t/0"},{"type":"function","doc":"The function returning the transformed input to be used with `Inspect` protocol.\n\nThis protocol is explicitly handful when deeply nested structs are to be logged/inspected.\n\n```elixir\niex|%_{}|1 ▶ %Estructura.User{}\n%Estructura.User{\n  address: %Estructura.User.Address{\n    city: nil,\n    street: %Estructura.User.Address.Street{house: nil, name: []}\n  },\n  data: %Estructura.User.Data{age: nil},\n  name: nil\n}\n\n\niex|%_{}|2 ▶ Estructura.Transformer.transform %Estructura.User{}, type: false, except: ~w|address.street data.age|\n[address: [city: nil], data: [], name: nil]\niex|%_{}|3 ▶ Estructura.Transformer.transform %Estructura.User{}, type: false, only: ~w|address.street data.age|\n[address: [street: [house: nil, name: []]], data: [age: nil]]\niex|%_{}|4 ▶ Estructura.Transformer.transform %Estructura.User{}, only: ~w|address|\n[\n  *: Estructura.User,\n  address: [\n    *: Estructura.User.Address,\n    city: nil,\n    street: [*: Estructura.User.Address.Street, house: nil, name: []]\n  ]\n]\n```\n\nTo enable it for your struct, use `@derive Estructura.Transformer` or\n  `@derive {Estructura.Transformer, options}`. `Estructura` implementations derive it be default.","title":"Estructura.Transformer.transform/2","ref":"Estructura.Transformer.html#transform/2"},{"type":"module","doc":"Additional generators to be used with `__generator__/1` generated by `Use Estructura`","title":"Estructura.StreamData","ref":"Estructura.StreamData.html"},{"type":"function","doc":"Generates an instance of `Date.t()`. This generator is unshrinkable.","title":"Estructura.StreamData.date/1","ref":"Estructura.StreamData.html#date/1"},{"type":"function","doc":"Generates an instance of `DateTime.t()`. This generator is unshrinkable.","title":"Estructura.StreamData.datetime/1","ref":"Estructura.StreamData.html#datetime/1"},{"type":"function","doc":"Identity function as a stream data generator","title":"Estructura.StreamData.id/1","ref":"Estructura.StreamData.html#id/1"},{"type":"function","doc":"Generates an instance of `Estructura.Nested.Type.IP.t()`. This generator is unshrinkable.","title":"Estructura.StreamData.ip4/1","ref":"Estructura.StreamData.html#ip4/1"},{"type":"function","doc":"Generates an instance of `Estructura.Nested.Type.IP.t()`. This generator is unshrinkable.","title":"Estructura.StreamData.ip6/1","ref":"Estructura.StreamData.html#ip6/1"},{"type":"function","doc":"Generates an instance of `Estructura.Nested.Type.IP.t()`. This generator is unshrinkable.","title":"Estructura.StreamData.ip/1","ref":"Estructura.StreamData.html#ip/1"},{"type":"function","doc":"Helper to generate _unshrinkable_ streams as per `Stream.iterate/2`.","title":"Estructura.StreamData.iterate/2","ref":"Estructura.StreamData.html#iterate/2"},{"type":"opaque","doc":"An opaque type that represents an `Estructura.StreamData` generator that generates values\nof type `a`.","title":"Estructura.StreamData.t/1","ref":"Estructura.StreamData.html#t:t/1"},{"type":"function","doc":"Generates an instance of `Time.t()`. This generator is unshrinkable.","title":"Estructura.StreamData.time/1","ref":"Estructura.StreamData.html#time/1"},{"type":"function","doc":"Generates an instance of `URI.t()`. This generator is unshrinkable.\n\n_See:_ `URI`.\n\n```elixir\n[scheme]://[userinfo]@[host]:[port][path]?[query]#[fragment]\n```","title":"Estructura.StreamData.uri/1","ref":"Estructura.StreamData.html#uri/1"},{"type":"module","doc":"The field stub allowing lazy instantiation of `Estructura` fields.","title":"Estructura.Lazy","ref":"Estructura.Lazy.html"},{"type":"function","doc":"Apply the lazy getter to the data passed as an argument","title":"Estructura.Lazy.apply/3","ref":"Estructura.Lazy.html#apply/3"},{"type":"function","doc":"iex> lazy = Estructura.Lazy.new(&System.fetch_env/1)\n    ...> Estructura.Lazy.apply(lazy, \"LANG\").value\n    System.fetch_env(\"LANG\")","title":"Examples - Estructura.Lazy.apply/3","ref":"Estructura.Lazy.html#apply/3-examples"},{"type":"type","doc":"","title":"Estructura.Lazy.cached/0","ref":"Estructura.Lazy.html#t:cached/0"},{"type":"type","doc":"","title":"Estructura.Lazy.getter/0","ref":"Estructura.Lazy.html#t:getter/0"},{"type":"type","doc":"","title":"Estructura.Lazy.key/0","ref":"Estructura.Lazy.html#t:key/0"},{"type":"function","doc":"Create the new struct with the getter passed as an argument","title":"Estructura.Lazy.new/2","ref":"Estructura.Lazy.html#new/2"},{"type":"function","doc":"Validates if the value is not stale yet according to `expires_in` setting","title":"Estructura.Lazy.stale?/1","ref":"Estructura.Lazy.html#stale?/1"},{"type":"type","doc":"","title":"Estructura.Lazy.t/0","ref":"Estructura.Lazy.html#t:t/0"},{"type":"type","doc":"","title":"Estructura.Lazy.value/0","ref":"Estructura.Lazy.html#t:value/0"},{"type":"module","doc":"The implementation of lazy map implementing lazy `Access` for its keys.\n\n`Estructura.LazyMap` is backed by the “raw” object and a key-value pairs\n  where values might be instances of `Estructura.Lazy`. If this is a case,\n  they will be accessed through `Lazy` implementation.\n\nValues might be also raw values, which makes `LazyMap` a drop-in replacement\n  of standard _Elixir_ maps, assuming they are accessed through `Access`\n  only (e. g. `map[:key]` and not `map.key`.)","title":"Estructura.LazyMap","ref":"Estructura.LazyMap.html"},{"type":"function","doc":"Eagerly instantiates the data","title":"Estructura.LazyMap.fetch_all/1","ref":"Estructura.LazyMap.html#fetch_all/1"},{"type":"function","doc":"Returns all the keys of the underlying map","title":"Estructura.LazyMap.keys/1","ref":"Estructura.LazyMap.html#keys/1"},{"type":"function","doc":"Creates new instance of `LazyMap` with a second parameter being a backed up object,\n  which would be used for lazy retrieving data for values, when value is an instance\n  of `Estructura.Lazy`.","title":"Estructura.LazyMap.new/2","ref":"Estructura.LazyMap.html#new/2"},{"type":"function","doc":"iex> lm = Estructura.LazyMap.new(\n    ...>   [int: Estructura.Lazy.new(&Estructura.LazyInst.parse_int/1)], \"42\")\n    ...> get_in lm, [:int]\n    42","title":"Examples - Estructura.LazyMap.new/2","ref":"Estructura.LazyMap.html#new/2-examples"},{"type":"type","doc":"","title":"Estructura.LazyMap.t/0","ref":"Estructura.LazyMap.html#t:t/0"},{"type":"behaviour","doc":"Behaviour for coercion delegates. Instead of implementing the coercion handlers\n  in `Estructura.Nested` inplace, one might do\n    \n```elixir\ncoerce do\n  defdelegate foo.bar.created_at(value), to: :date\nend\n```","title":"Estructura.Coercer","ref":"Estructura.Coercer.html"},{"type":"callback","doc":"","title":"Estructura.Coercer.coerce/1","ref":"Estructura.Coercer.html#c:coerce/1"},{"type":"module","doc":"Default coercer for `:date`, coercing strings (_ISO8601_) and integers (_epoch_)","title":"Estructura.Coercers.Date","ref":"Estructura.Coercers.Date.html"},{"type":"module","doc":"Default coercer for `:datetime`, coercing strings (_ISO8601_) and integers (_epoch_)","title":"Estructura.Coercers.Datetime","ref":"Estructura.Coercers.Datetime.html"},{"type":"module","doc":"Default coercer for `:float`, coercing strings and integers by multiplying by `1.0`","title":"Estructura.Coercers.Float","ref":"Estructura.Coercers.Float.html"},{"type":"module","doc":"Default coercer for `:integer`, coercing strings and floats by rounding","title":"Estructura.Coercers.Integer","ref":"Estructura.Coercers.Integer.html"},{"type":"module","doc":"Nullable coercer for `:date`, coercing strings and floats by rounding, allows `nil` value","title":"Estructura.Coercers.NullableDate","ref":"Estructura.Coercers.NullableDate.html"},{"type":"module","doc":"Nullable coercer for `:datetime`, coercing strings and floats by rounding, allows `nil` value","title":"Estructura.Coercers.NullableDatetime","ref":"Estructura.Coercers.NullableDatetime.html"},{"type":"module","doc":"Nullable coercer for `:float`, coercing strings and floats by rounding, allows `nil` value","title":"Estructura.Coercers.NullableFloat","ref":"Estructura.Coercers.NullableFloat.html"},{"type":"module","doc":"Nullable coercer for `:integer`, coercing strings and floats by rounding, allows `nil` value","title":"Estructura.Coercers.NullableInteger","ref":"Estructura.Coercers.NullableInteger.html"},{"type":"module","doc":"Nullable coercer for `:time`, coercing strings and floats by rounding, allows `nil` value","title":"Estructura.Coercers.NullableTime","ref":"Estructura.Coercers.NullableTime.html"},{"type":"module","doc":"Default coercer for `:time`, coercing strings (_ISO8601_) and integers (_epoch_)","title":"Estructura.Coercers.Time","ref":"Estructura.Coercers.Time.html"},{"type":"module","doc":"The configuration of the `Estructura` that is built from the parameters,\npassed as a second argument in a call to `use Estructura`.\n\nOne normally does not need to meddle with this module.","title":"Estructura.Config","ref":"Estructura.Config.html"},{"type":"type","doc":"The generator to be passed to `use Estructura` should be given in one of these forms","title":"Estructura.Config.generator/0","ref":"Estructura.Config.html#t:generator/0"},{"type":"type","doc":"The structure key","title":"Estructura.Config.key/0","ref":"Estructura.Config.html#t:key/0"},{"type":"type","doc":"The configuration of `Estructura`","title":"Estructura.Config.t/0","ref":"Estructura.Config.html#t:t/0"},{"type":"type","doc":"The structure value","title":"Estructura.Config.value/0","ref":"Estructura.Config.html#t:value/0"},{"type":"module","doc":"Full Example","title":"Estructura.Full","ref":"Estructura.Full.html"},{"type":"function","doc":"See `Estructura.Full.__generator__/1`","title":"Estructura.Full.__generator__/0","ref":"Estructura.Full.html#__generator__/0"},{"type":"function","doc":"Returns the generator to be used in `StreamData`-powered property testing, based\n  on the specification given to `use Estructura`, which contained","title":"Estructura.Full.__generator__/1","ref":"Estructura.Full.html#__generator__/1"},{"type":"function","doc":"```elixir\n%Estructura.Config{\n  access: true,\n  coercion: [:foo],\n  validation: true,\n  calculated: [],\n  collectable: :bar,\n  enumerable: true,\n  generator: [\n    foo: {StreamData, :integer, [0..1000]},\n    bar: {StreamData, :string, [:alphanumeric]},\n    baz: {StreamData, :fixed_map,\n     [[key1: {StreamData, :integer}, key2: {StreamData, :integer}]]},\n    zzz: &Estructura.Full.zzz_generator/0\n  ]\n}\n```\n\n\nThe argument given would be used as a template to generate new values.","title":"shape - Estructura.Full.__generator__/1","ref":"Estructura.Full.html#__generator__/1-shape"},{"type":"function","doc":"Gets the value for the given key from the structure","title":"Estructura.Full.get/3","ref":"Estructura.Full.html#get/3"},{"type":"function","doc":"Puts the value for the given key into the structure, passing coercion _and_ validation,\n  returns `{:ok, updated_struct}` or `{:error, reason}` if there is no such key","title":"Estructura.Full.put/3","ref":"Estructura.Full.html#put/3"},{"type":"function","doc":"Puts the value for the given key into the structure, passing coercion _and_ validation,\n  returns the value or raises if there is no such key","title":"Estructura.Full.put!/3","ref":"Estructura.Full.html#put!/3"},{"type":"function","doc":"","title":"Estructura.Full.recalculate_calculated/1","ref":"Estructura.Full.html#recalculate_calculated/1"},{"type":"module","doc":"Nested example. The source code of the file follows.\n\n```elixir\n\ndefmodule Calculated do\n  @moduledoc false\n  def person(this) do\n    this.name <> \", \" <> this.address.city\n  end\nend\n\nuse Estructura.Nested, calculated: [person: &Calculated.person/1]\n\nshape %{\n  created_at: :datetime,\n  name: :string,\n  address: %{city: :string, street: %{name: [:string], house: :string}},\n  person: :string,\n  data: %{age: :float},\n  birthday: Estructura.Nested.Type.Date,\n  title: {Estructura.Nested.Type.Enum, ~w|junior middle señor|},\n  tags: {Estructura.Nested.Type.Tags, ~w|backend frontend|}\n}\n\ninit %{\n  name: \"Aleksei\",\n  address: %{city: \"Barcelona\"}\n}\n\ncoerce do\n  def data.age(age) when is_float(age), do: {:ok, age}\n  def data.age(age) when is_integer(age), do: {:ok, 1.0 * age}\n  def data.age(age) when is_binary(age) do\n    age\n    |> Float.parse()\n    |> case do\n      {age, \"\"} -> {:ok, age}\n      {age, _rest} -> {:ok, age}\n      :error -> {:ok, 0.0}\n    end\n  end\nend\n\ncoerce do\n  def name(value) when is_binary(value), do: {:ok, value}\n  def name(value) when is_atom(value), do: {:ok, Atom.to_string(value)}\nend\n\nvalidate do\n  def address.street.house(house), do: {:ok, house}\nend\n```\n\nNow one can cast it from map as below\n\n```elixir\nUser.cast %{\n  address: %{city: \"London\", street: %{name: \"Baker\", house: \"221 Bis\"}},\n  data: %{age: 32},\n  name: \"Watson\",\n  birthday: \"1973-09-30\",\n  title: \"señor\",\n  tags: [\"backend\"]}\n\n{:ok,\n   %Estructura.User{\n     address: %Estructura.User.Address{\n       city: \"London\",\n       street: %Estructura.User.Address.Street{house: \"221 Bis\", name: \"Baker\"}\n     },\n     data: %Estructura.User.Data{age: 32.0},\n     name: \"Watson\",\n     person: \"Watson, London\",\n     birthday: ~D[1973-09-30],\n     title: \"señor\",\n     tags: [\"backend\"]}\n```","title":"Estructura.User","ref":"Estructura.User.html"},{"type":"function","doc":"See `Estructura.User.__generator__/1`","title":"Estructura.User.__generator__/0","ref":"Estructura.User.html#__generator__/0"},{"type":"function","doc":"Returns the generator to be used in `StreamData`-powered property testing, based\n  on the specification given to `use Estructura.Nested`, which contained","title":"Estructura.User.__generator__/1","ref":"Estructura.User.html#__generator__/1"},{"type":"function","doc":"```elixir\n%{\n  data: %{age: :float},\n  name: :string,\n  address: %{city: :string, street: %{name: [:string], house: :string}},\n  title: {Estructura.Nested.Type.Enum, [\"junior\", \"middle\", \"señor\"]},\n  created_at: :datetime,\n  birthday: Estructura.Nested.Type.Date,\n  tags: {Estructura.Nested.Type.Tags, [\"backend\", \"frontend\"]},\n  person: :string\n}\n```","title":"shape - Estructura.User.__generator__/1","ref":"Estructura.User.html#__generator__/1-shape"},{"type":"function","doc":"```elixir\n  def coerce_age(age) when is_float(age) do\n    {:ok, age}\n  end\n  def coerce_age(age) when is_integer(age) do\n    {:ok, 1.0 * age}\n  end\n  def coerce_age(age) when is_binary(age) do\n    age\n    |> Float.parse()\n    |> case do\n      {age, \"\"} -> {:ok, age}\n      {age, _rest} -> {:ok, age}\n      :error -> {:ok, 0.0}\n    end\n  end\n  def coerce_created_at(value) do\n    Estructura.Coercers.Datetime.coerce(value)\n  end\n  def coerce_name(value) when is_binary(value) do\n    {:ok, value}\n  end\n  def coerce_name(value) when is_atom(value) do\n    {:ok, Atom.to_string(value)}\n  end\n```","title":"coerce - Estructura.User.__generator__/1","ref":"Estructura.User.html#__generator__/1-coerce"},{"type":"function","doc":"```elixir\n  def validate_house(house) do\n    {:ok, house}\n  end\n```\n\n\nThe argument given would be used as a template to generate new values.","title":"validate - Estructura.User.__generator__/1","ref":"Estructura.User.html#__generator__/1-validate"},{"type":"function","doc":"Casts the map representation as given to `Estructura.Nested.shape/1` to\n  the nested `Estructura` instance.\n\nIf `split: true` is passed as an option, it will attempt to put `foo_bar` into nested `%{foo: %{bar: _}}`","title":"Estructura.User.cast/2","ref":"Estructura.User.html#cast/2"},{"type":"function","doc":"","title":"Estructura.User.cast!/2","ref":"Estructura.User.html#cast!/2"},{"type":"function","doc":"Gets the value for the given key from the structure","title":"Estructura.User.get/3","ref":"Estructura.User.html#get/3"},{"type":"function","doc":"Safely parses the json, applying all the specified validations and coercions","title":"Estructura.User.parse/1","ref":"Estructura.User.html#parse/1"},{"type":"function","doc":"Same as `parse/1` but either returns the result of successful parsing or raises","title":"Estructura.User.parse!/1","ref":"Estructura.User.html#parse!/1"},{"type":"function","doc":"Puts the value for the given key into the structure, passing coercion _and_ validation,\n  returns `{:ok, updated_struct}` or `{:error, reason}` if there is no such key","title":"Estructura.User.put/3","ref":"Estructura.User.html#put/3"},{"type":"function","doc":"Puts the value for the given key into the structure, passing coercion _and_ validation,\n  returns the value or raises if there is no such key","title":"Estructura.User.put!/3","ref":"Estructura.User.html#put!/3"},{"type":"function","doc":"","title":"Estructura.User.recalculate_calculated/1","ref":"Estructura.User.html#recalculate_calculated/1"},{"type":"type","doc":"","title":"Estructura.User.t/0","ref":"Estructura.User.html#t:t/0"},{"type":"extras","doc":"# Estructura    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  [![Test](https://github.com/am-kantox/estructura/workflows/Test/badge.svg)](https://github.com/am-kantox/estructura/actions?query=workflow%3ATest)  [![Dialyzer](https://github.com/am-kantox/estructura/workflows/Dialyzer/badge.svg)](https://github.com/am-kantox/estructura/actions?query=workflow%3ADialyzer)\n\n**Extensions for _Elixir_ structures.**","title":"Estructura","ref":"readme.html"},{"type":"extras","doc":"```elixir\ndef deps do\n  [\n    {:estructura, \"~> 0.1\"},\n    # optionally you might want to add `boundary` library \n    # it is used by `estructura` and many other projects\n    # more info: https://hexdocs.pm/boundary\n    {:boundary, \"~> 0.9\", runtime: false}\n  ]\nend\n```\nI suggest adding [`boundary`](https://hexdocs.pm/boundary) as a dependency since that is used in this project.","title":"Installation - Estructura","ref":"readme.html#installation"},{"type":"extras","doc":"* `1.7.0` — better infrastructure for `Types`, `URI`, `IP`, `Scaffold`\n* `1.6.0` — `jsonify: true | module()` option in a call to `Estructura.Flattenable.flatten/2`\n* `1.5.0` — no `:formulae` dependency\n* `1.4.1` — allow functions of arity 1 in `content` as coercers in a call to `Estructura.Aston.coerce/2`\n* `1.4.0` — allow coercers in a call to `Estructura.Aston.coerce/2`\n* `1.3.0` — calculated fields for `Estructura` and `Estructura.Nested`\n* `1.2.12` — export type from `Estructura.Nested`\n* `1.2.11` — nullable coercers\n* `1.2.10` — coercers for floats, and date/time values\n* `1.2.8` — `Estructura.Tree` → `Estructura.Aston` + `Aston.access/2` to retrieve and access key by names\n* `1.2.5` — `use Estructura.Nested flattenable: boolean(), jason: boolean(), transformer: boolean()`\n* `1.2.3` — Several `coerce/1` and `validate/1` clauses, default coercers\n* `1.2.2` — `Estructura.Flattenable`\n* `1.2.1` — Generators for `:datetime` and `:date`\n* `1.2.0` — `Estructura.Nested` would attempt to split keys by a delimiter if instructed\n* `1.1.0` — `Estructura.Aston` to hold an AST structure, like XML\n* `1.0.0` — Elixir v1.16 and deps\n* `0.6.0` — `Estructura.Transform` to produce squeezed representations of nested structs\n* `0.5.5` — export declarations of both `Estructura` and `Estructura.Nested` to docs\n* `0.5.4` — `Estructura.Nested` allows `cast/1` to cast nested structs from maps\n* `0.5.3` — `Estructura.diff/3` now understands maps\n* `0.5.2` — `Estructura.diff/3`\n* `0.5.1` — [BUG] Fixed `Collectable` and `Enumerable` injected implementations\n* `0.5.0` — `Estructura.Nested` for nested structures with validation, coercion, and generation\n* `0.4.2` — [BUG] Fixed wrong spec for `put!/3`\n* `0.4.1` — `Estructura.LazyMap.keys/1`, `Estructura.LazyMap.fetch_all/1`\n* `0.4.0` — `Estructura.Lazy`, `Estructura.LazyMap`\n* `0.3.2` — `put!/3`\n* `0.3.0` — `coercion` and `validation` are now injected as behaviours\n* `0.2.0` — `coercion`, `validation`, `put/3`\n\n## [Documentation](https://hexdocs.pm/estructura)","title":"Changelog - Estructura","ref":"readme.html#changelog"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}